@@ -12,6 +12,7 @@ import os
 import zipfile
 
 import basico.model_info
+from basico.callbacks import get_default_handler
 
 try:
     import urllib2
@@ -215,10 +216,10 @@ def load_model_from_string(content):
     if '<COPASI ' in content and model.loadModelFromString(content, os.getcwd()):
         return set_current_model(model)
 
-    if '<sbml ' in content and model.importSBMLFromString(content):
+    if '<sbml ' in content and model.importSBMLFromString(content, pImportHandler=get_default_handler()):
         return set_current_model(model)
 
-    if '<sedML '  in content and model.importSEDMLFromString(content):
+    if '<sedML '  in content and model.importSEDMLFromString(content, pImportHandler=get_default_handler()):
         return set_current_model(model)
 
     return remove_datamodel(model)
@@ -252,14 +253,15 @@ def load_model(location):
     model = create_datamodel()
 
     if os.path.isfile(location):
+        location = os.path.abspath(location)
         if zipfile.is_zipfile(location):
-            if model.openCombineArchive(location):
+            if model.openCombineArchive(location, pProgressReport=get_default_handler()):
                 return set_current_model(model)
-        if model.importSBML(location):
+        if model.importSBML(location, pImportHandler=get_default_handler()):
             return set_current_model(model)
         if model.loadModel(location):
             return set_current_model(model)
-        if model.importSEDML(location):
+        if model.importSEDML(location, pImportHandler=get_default_handler()):
             return set_current_model(model)
 
     remove_datamodel(model)
